package de.hottis.measurementCollector;

import de.hottis.common.JmsTopic;
import de.hottis.smarthomelib.ADataObject;
import java.io.FileInputStream;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


public class MeasurementCollector {
	static final String PROPS_FILENAME = "measurementCollector.props";
	static final Logger logger = LogManager.getRootLogger();
	
	
	public static void main(String[] args) throws Exception {
		logger.info("MeasurementCollector starting");
		
		final Properties config = new Properties();
		try (FileInputStream propsFileInputStream = new FileInputStream(PROPS_FILENAME)) {
			config.load(propsFileInputStream);
		}
		logger.debug("Configuration loaded");
		
		MqttReceiver mqttReceiver = new MqttReceiver(config);
		logger.debug("MqttReceiver started");
		
		JmsTopic<ADataObject> queue = new JmsTopic<>(config, JmsTopic.Mode.PRODUCER); 
		queue.init();
		
		MBusParser mbusParser = new MBusParser(config, queue);
		mbusParser.init();
		mbusParser.registerConfiguredDataParsers();
		mqttReceiver.registerParser(mbusParser);
		logger.debug("MBusParser started");

                EspThermometerParser espThermometerParser = new EspThermometerParser(config, queue);
                espThermometerParser.init();
                mqttReceiver.registerParser(espThermometerParser);
                logger.debug("EspThermometerParser started");

		mqttReceiver.connect();
		logger.debug("MqttReceiver connected");
        }
		
}
